@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 2500 2007-06-25 19:05:46Z wwalker $"
-__version__   = "$Revision: 2500 $"
-__date__      = "$Date: 2007-06-25 15:05:46 -0400 (Mon, 25 Jun 2007) $"
+__id__        = "$Id: flat_review.py 2718 2007-08-26 18:49:08Z joanied $"
+__version__   = "$Revision: 2718 $"
+__date__      = "$Date: 2007-08-26 14:49:08 -0400 (Sun, 26 Aug 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -37,6 +37,7 @@ import rolenames
 import settings
 
 from orca_i18n import _         # for gettext support
+from orca_i18n import Q_        # to provide qualified translatable strings
 
 # [[[WDW - HACK Regular expression to split strings on whitespace
 # boundaries, which is what we'll use for word dividers instead of
@@ -324,12 +325,16 @@ class StateZone(Zone):
                     # Translators: this is in reference to a radio button being
                     # selected or not.
                     #
-                    speechState = _("selected")
+                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                    #
+                    speechState = Q_("radiobutton|selected")
                 else:
                     # Translators: this is in reference to a radio button being
                     # selected or not.
                     #
-                    speechState = _("not selected")
+                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                    #
+                    speechState = Q_("radiobutton|not selected")
                 brailleState = \
                     settings.brailleRadioButtonIndicators[stateCount]
 
@@ -359,6 +364,23 @@ class ValueZone(Zone):
 
     def __getattr__(self, attr):
         if attr in ["string", "length", "brailleString"]:
+            orientation = None
+            if self.accessible.role in [rolenames.ROLE_SLIDER,
+                                        rolenames.ROLE_SCROLL_BAR]:
+                horizontalCount = \
+                    self.accessible.state.count(atspi.Accessibility.STATE_HORIZONTAL)
+                if horizontalCount:
+                    # Translators: The component orientation is horizontal.
+                    #
+                    orientation = _("horizontal")
+                else:
+                    verticalCount = \
+                        self.accessible.state.count(atspi.Accessibility.STATE_VERTICAL)
+                    if verticalCount:
+                        # Translators: The component orientation is vertical.
+                        #
+                        orientation = _("vertical")
+                        
             value = self.accessible.value
             currentValue = int(value.currentValue)
             percentValue = int((value.currentValue
@@ -366,14 +388,26 @@ class ValueZone(Zone):
                                    - value.minimumValue))
                                * 100.0)
 
-            # Translators: this is the percentage value of a
-            # progress bar.
+            if orientation:
+                speechValue = orientation + " " + \
+                              rolenames.getSpeechForRoleName(self.accessible)
+            else:
+                speechValue = rolenames.getSpeechForRoleName(self.accessible)
+                
+            # Translators: this is the percentage value of a slider, progress bar
+            # or other component that displays a value as a percentage.
             #
-            speechValue = (rolenames.getSpeechForRoleName(self.accessible) \
-                           + " " + _("%d percent.") % percentValue)
-            brailleValue = "%s %d%%" \
-                           % (rolenames.getBrailleForRoleName(self.accessible),
-                              percentValue)
+            speechValue = speechValue + " " + _("%d percent.") % percentValue
+
+            if orientation:
+                brailleValue = "%s %s %d%%" \
+                               % (orientation,
+                                  rolenames.getBrailleForRoleName(self.accessible),                                  
+                                  percentValue)
+            else:
+                brailleValue = "%s %d%%" \
+                               % (rolenames.getBrailleForRoleName(self.accessible),
+                                  percentValue)
 
             if attr == "string":
                 return speechValue
@@ -1042,22 +1076,22 @@ class Context:
                              cliprect.x, cliprect.y,
                              cliprect.width, cliprect.height)
 
-        if (accessible.role != rolenames.ROLE_COMBO_BOX) \
+        if (len(zones) == 0) \
+            and accessible.role in [rolenames.ROLE_SCROLL_BAR,
+                                    rolenames.ROLE_SLIDER,
+                                    rolenames.ROLE_PROGRESS_BAR]:
+            zones.append(ValueZone(accessible,
+                                   clipping[0],
+                                   clipping[1],
+                                   clipping[2],
+                                   clipping[3]))
+        elif (accessible.role != rolenames.ROLE_COMBO_BOX) \
             and (accessible.role != rolenames.ROLE_EMBEDDED) \
             and (accessible.role != rolenames.ROLE_LABEL) \
             and (accessible.role != rolenames.ROLE_MENU) \
             and (accessible.role != rolenames.ROLE_PAGE_TAB) \
             and accessible.childCount > 0:
             pass
-        elif (len(zones) == 0) \
-             and accessible.role in [rolenames.ROLE_SCROLL_BAR,
-                                     rolenames.ROLE_SLIDER,
-                                     rolenames.ROLE_PROGRESS_BAR]:
-            zones.append(ValueZone(accessible,
-                                   clipping[0],
-                                   clipping[1],
-                                   clipping[2],
-                                   clipping[3]))
         elif len(zones) == 0:
             if accessible.name and len(accessible.name):
                 string = accessible.name
@@ -1210,7 +1244,8 @@ class Context:
         if (root.parent and (root.parent.role == rolenames.ROLE_MENU_BAR)) \
            or (root.role == rolenames.ROLE_COMBO_BOX) \
            or (root.role == rolenames.ROLE_EMBEDDED) \
-           or (root.role == rolenames.ROLE_TEXT):
+           or (root.role == rolenames.ROLE_TEXT) \
+           or (root.role == rolenames.ROLE_SCROLL_BAR):
             return self.getZonesFromAccessible(root, root.extents)
 
         # Otherwise, dig deeper.
