@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2828 2007-09-13 19:26:33Z wwalker $"
-__version__   = "$Revision: 2828 $"
-__date__      = "$Date: 2007-09-13 15:26:33 -0400 (Thu, 13 Sep 2007) $"
+__id__        = "$Id: focus_tracking_presenter.py 2821 2007-09-12 14:28:07Z wwalker $"
+__version__   = "$Revision: 2821 $"
+__date__      = "$Date: 2007-09-12 10:28:07 -0400 (Wed, 12 Sep 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -572,7 +572,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         oldLocusOfFocus = orca_state.locusOfFocus
         try:
             # If we've received a mouse event, then don't try to get
-            # event.source.app because the top most parent is of role 
+            # event.source.app because the top most parent is of role
             # unknown, which will cause an ERROR message to be displayed.
             # See Orca bug #409731 for more details.
             #
@@ -661,17 +661,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - e: an at-spi event.
         """
 
-        # Uncomment these lines if you want to see what it's like without
-        # the queue.
-        #
-        #if isinstance(e, input_event.KeyboardEvent):
-        #    self._processKeyboardEvent(e)
-        #elif isinstance(e, input_event.BrailleEvent):
-        #    self._processBrailleEvent(e)
-        #else:
-        #    self._processObjectEvent(atspi.Event(e))
-        #return
-
         if settings.debugEventQueue:
             if self._enqueueEventCount:
                 debug.println(debug.LEVEL_ALL,
