@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 3032 2007-10-12 19:31:00Z richb $"
-__version__   = "$Revision: 3032 $"
-__date__      = "$Date: 2007-10-12 15:31:00 -0400 (Fri, 12 Oct 2007) $"
+__id__        = "$Id: orca.py 2953 2007-10-01 20:02:55Z wwalker $"
+__version__   = "$Revision: 2953 $"
+__date__      = "$Date: 2007-10-01 16:02:55 -0400 (Mon, 01 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -45,7 +45,6 @@ except:
     pass
 
 import getopt
-import gnome
 import os
 import signal
 import time
@@ -1058,19 +1057,6 @@ def _showPreferencesConsole(script=None, inputEvent=None):
 
     return True
 
-def helpForOrca(script=None, inputEvent=None):
-    """Show Orca Help window (part of the GNOME Access Guide).
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    props = { gnome.PARAM_APP_DATADIR : '/usr/share'}
-    prog = gnome.program_init('orca', '1.0', properties=props)
-    gnome.help_display_with_doc_id(prog, 'gnome-access-guide', 
-                                         'gnome-access-guide', 'ats-2')
-
-    return True
-
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
