@@ -33,9 +33,9 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 2822 2007-09-12 16:11:37Z wwalker $"
-__version__   = "$Revision: 2822 $"
-__date__      = "$Date: 2007-09-12 12:11:37 -0400 (Wed, 12 Sep 2007) $"
+__id__        = "$Id: script.py 2936 2007-09-27 19:46:45Z eitani $"
+__version__   = "$Revision: 2936 $"
+__date__      = "$Date: 2007-09-27 15:46:45 -0400 (Thu, 27 Sep 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -77,6 +77,7 @@ class Script:
         self.presentIfInactive = True
 
         self.inputEventHandlers = {}
+        self.pointOfReference = {}
         self.setupInputEventHandlers()
         self.keyBindings = self.getKeyBindings()
         self.brailleBindings = self.getBrailleBindings()
@@ -98,8 +99,8 @@ class Script:
         # giving up on an object.  This might need to be overridden by the
         # script.  See bug #397787.
         #
-        self.commFailureWaitTime = 0.1
-        self.commFailureAttemptLimit = 5
+        self.commFailureWaitTime = settings.commFailureWaitTime
+        self.commFailureAttemptLimit = settings.commFailureAttemptLimit
 
         debug.println(debug.LEVEL_FINE, "NEW SCRIPT: %s" % self.name)
 
