@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 2988 2007-10-07 15:42:30Z joanied $"
-__version__   = "$Revision: 2988 $"
-__date__      = "$Date: 2007-10-07 11:42:30 -0400 (Sun, 07 Oct 2007) $"
+__id__        = "$Id: speechgenerator.py 2959 2007-10-01 22:30:24Z richb $"
+__version__   = "$Revision: 2959 $"
+__date__      = "$Date: 2007-10-01 18:30:24 -0400 (Mon, 01 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -738,7 +738,7 @@ class SpeechGenerator:
                              utterances)
 
         return utterances
-
+    
     def _getSpeechForListItem(self, obj, already_focused):
         """Get the speech for a listitem.
 
@@ -751,6 +751,7 @@ class SpeechGenerator:
 
         utterances = self._getDefaultSpeech(obj, already_focused)
 
+        # If already in focus then the tree probably collapsed or expanded
         if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
             if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
                 # Translators: this represents the state of a node in a tree.
@@ -765,6 +766,9 @@ class SpeechGenerator:
                 #
                 utterances.append(_("collapsed"))
                 
+        if not already_focused:
+            utterances.append(rolenames.getSpeechForRoleName(obj))
+
         self._debugGenerator("_getSpeechForListItem",
                              obj,
                              already_focused,
@@ -1331,7 +1335,7 @@ class SpeechGenerator:
                           self._script.getRealActiveDescendant(obj))
         utterances.append(displayedText)
 
-        # If there is no displayed text, check to see if this table cell
+        # If there is no displayed text, check to see if this table cell 
         # contains an icon (image). If yes:
         #   1/ Try to get a description for it and speak that.
         #   2/ Treat the object of role type ROLE_IMAGE and speak
@@ -1405,11 +1409,12 @@ class SpeechGenerator:
                 # the same row).
                 #
                 speakAll = True
-                if parent.__dict__.has_key("lastRow") and \
-                    parent.__dict__.has_key("lastColumn"):
-                    speakAll = (parent.lastRow != row) or \
-                           ((row == 0 or row == parent.table.nRows-1) and \
-                            parent.lastColumn == column)
+                if self._script.pointOfReference.has_key("lastRow") and \
+                    self._script.pointOfReference.has_key("lastColumn"):
+                    pointOfReference = self._script.pointOfReference
+                    speakAll = (pointOfReference["lastRow"] != row) or \
+                        ((row == 0 or row == parent.table.nRows-1) and \
+                           pointOfReference["lastColumn"] == column)
 
                 if speakAll:
                     for i in range(0, parent.table.nColumns):
